local inCombat = nil
local dndBanner = "<SBW> "

local dndString = dndBanner .. "I'm busy fighting %s (currently at %d%% with %d/%d players alive). You'll be notified when combat ends."
local combatEndedString = dndBanner .. "Combat ended after %d minutes."

do
	local locale = GetLocale()
	if locale == "deDE" then
		dndString = dndBanner .. "Ich bin im Kampf mit %s (noch %d%% Leben mit %d/%d Spielern am Leben). Du wirst nach Kampfende benachrichtigt werden."
		combatEndedString = dndBanner .. "Kampf beendet nach %d Minuten."
	elseif locale == "zhTW" then
		dndString = dndBanner .. "正與 %s 戰鬥中 (目前尚有 %d%% / %d/%d 存活者),戰鬥結束後將會提示你."
		combatEndedString = dndBanner .. "戰鬥已結束,花費時間: %d 分鐘"
	end
end

local whisperers = {}
local combatStart = nil
local totalElapsed = 0

local bossName = nil
local bossHp = nil

local possibleTargets = nil
local function buildTable()
	possibleTargets = {
		"boss1", "boss2", "boss3", "boss4",
		"target", "targettarget",
		"focus", "focustarget",
		"mouseover", "mouseovertarget",
		"party1target", "party2target", "party3target", "party4target"
	}
	for i = 1, 25 do possibleTargets[#possibleTargets+1] = string.format("raid%dtarget", i) end
end
local function scan()
	if not possibleTargets then buildTable() end
	for i, id in next, possibleTargets do
		if UnitExists(id) and UnitAffectingCombat(id) and UnitClassification(id) == "worldboss" then return id end
	end
end

local frame = CreateFrame("Frame")
local function updateBossTarget()
	local target = scan()
	if not target then return end
	if not bossName then
		frame:RegisterEvent("UNIT_HEALTH")
	end
	bossName = UnitName(target)
	bossHp = math.floor(UnitHealth(target)/UnitHealthMax(target)*100+0.5)
end

local function onUpdate(self, elapsed)
	totalElapsed = totalElapsed + elapsed
	if totalElapsed > 1 then
		local t = scan()
		if t then totalElapsed = 0; return end
		self:SetScript("OnUpdate", nil)

		local time = GetTime() - combatStart
		local msg = combatEndedString:format(math.floor(time / 60))
		for k, v in pairs(whisperers) do
			-- Notify people that combat has ended
			SendChatMessage(msg, "WHISPER", nil, k)
			whisperers[k] = nil
		end
		self:UnregisterEvent("UNIT_HEALTH")
		self:UnregisterEvent("PLAYER_TARGET_CHANGED")

		combatStart = nil
		bossName = nil
		bossHp = nil
	end
end

function frame:PLAYER_REGEN_DISABLED()
	local _, t = IsInInstance()
	if t ~= "raid" then return end
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	self:SetScript("OnUpdate", nil)
	totalElapsed = 0
	combatStart = GetTime()
	updateBossTarget()
end

function frame:PLAYER_TARGET_CHANGED()
	updateBossTarget()
end

function frame:PLAYER_REGEN_ENABLED()
	if combatStart then
		self:SetScript("OnUpdate", onUpdate)
	end
end

function frame:UNIT_HEALTH(msg)
	if not msg or UnitName(msg) ~= bossName then return end
	bossHp = math.floor(UnitHealth(msg) / UnitHealthMax(msg) * 100 + 0.5)
	-- Allow new status whispers every 10%
	if bossHp % 10 == 0 then
		for k in pairs(whisperers) do
			whisperers[k] = 1
		end
	end
end

do
	local addonName = ...
	function frame:ADDON_LOADED(msg)
		if msg ~= addonName then return end
		self:UnregisterEvent("ADDON_LOADED")
		if not SBW_DisableChatFilter then
			local bannerTest = "^" .. dndBanner
			ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", function(self, event, msg)
				if msg:find(bannerTest) then return true end
			end)
		end
	end
end

do
	local pname = UnitName("player")
	function frame:CHAT_MSG_WHISPER(msg, sender, _, _, _, gm)
		if not bossName then return end

		if type(sender) ~= "string" or sender == pname or
		  (UnitExists("target") and UnitName("target") == sender) or UnitInRaid(sender) or
		  (type(gm) == "string" and gm == "GM") then return end

		if not whisperers[sender] or whisperers[sender] == 1 or msg == "status" then
			-- Let him know we are fighting a boss
			whisperers[sender] = 2

			local total = GetNumRaidMembers()
			local alive = 0
			for i = 1, total do
				if not UnitIsDeadOrGhost(GetRaidRosterInfo(i)) then alive = alive + 1 end
			end
			SendChatMessage(dndString:format(bossName, bossHp, alive, total), "WHISPER", nil, sender)
		end
	end
end

frame:SetScript("OnEvent", function(self, event, msg, ...) self[event](self, msg, ...) end)
frame:RegisterEvent("CHAT_MSG_WHISPER")
frame:RegisterEvent("PLAYER_REGEN_DISABLED")
frame:RegisterEvent("PLAYER_REGEN_ENABLED")
frame:RegisterEvent("ADDON_LOADED")

